#include "Chassis.h"
#include "gpio.h"
#include "user_math.h"
#include "bsp_buzzer.h"
#include "bsp_delay.h"






/*机器人坐标系，向前x，右y，上yaw
   不同于nuc (前x，左y，上yaw)                          */
/*          
               x
               |
                 --y


*/

static int8_t Chassis_SetCtrl(Chassis_t *c,CMD_t *ctrl){
	if (c == NULL) return CHASSIS_ERR_NULL; /*主结构体不能为空 */
  if (ctrl->CMD_CtrlType== c->chassis_ctrl.ctrl && ctrl->CMD_mode == c->chassis_ctrl.mode) return CHASSIS_OK; /*模式未改变直接返回*/
   c->chassis_ctrl.ctrl =ctrl->CMD_CtrlType ;
   c->chassis_ctrl.mode =ctrl->CMD_mode ;
   
   return CHASSIS_OK;
}    //设置控制模式


/*该函数用来更新can任务获得的电机反馈值*/

int8_t Chassis_UpdateFeedback(Chassis_t *c, const CAN_t *can) {
  if (c == NULL) return CHASSIS_ERR_NULL;
  if (can == NULL) return CHASSIS_ERR_NULL;
  for (uint8_t i = 0; i < 4; i++) 
 {
	c->motorfeedback.rotor_rpm3508[i] = can->motor.chassis_motor3508.as_array[i].rotor_speed;
	c->motorfeedback.rotor_current3508[i] = can->motor.chassis_motor3508.as_array[i].torque_current;
 }
 
 

  return CHASSIS_OK;
}


int8_t Chassis_init(Chassis_t *c,const Chassis_Param_t *param,float target_freq)
{
   if (c == NULL) return CHASSIS_ERR_NULL;

   c->param = param; 	/*初始化参数 */
	
		 for(int i =0 ; i < 4 ; i++)
	{
		   PID_init(&(c->pid.chassis_3508VecPID[i]),PID_POSITION_D,&(c->param->M3508_param));  //带D项滤波
	}

	    PID_init(&(c->pid.chassis_PICKWzPID_HIGN),PID_POSITION,&(c->param->chassis_PICKWzPID_HIGN_param));
	
		  PID_init(&(c->pid.chassis_PICKWzPID_LOW),PID_POSITION,&(c->param->chassis_PICKWzPID_LOW_param));

	
	    PID_init(&(c->pid.sick_CaliforYPID),PID_POSITION,&(c->param->Sick_CaliYparam));
	  
	    PID_init(&(c->pid.sick_CaliforXPID),PID_POSITION,&(c->param->Sick_CaliXparam));
	
	    LowPassFilter2p_Init(&(c->filled[0]),target_freq,80.0f);  //给角加速度做滤波
	
	
	    LowPassFilter2p_Init(&(c->filled[1]),target_freq,80.0f); //给w 做滤波
	
	    LowPassFilter2p_Init(&(c->filled[2]),target_freq,80.0f);  //给y做滤波
	
	    LowPassFilter2p_Init(&(c->filled[3]),target_freq,80.0f); //给x 做滤波
			
		//
	
	
	 return CHASSIS_OK;
}

fp32 gyro_kp=1.0f;
fp32 PIAN=0;
fp32 shang_yaw=0;
fp32 pian_yaw=0;

void Chassis_speed_calculate(Chassis_t *c,fp32 Vx,fp32 Vy,fp32 Vw)  //底盘逆运动学的解算,
{		
	  fp64 Nor_Vx,Nor_Vy;//归一化后的数据
	  normalize_vector(Vx,Vy,&Nor_Vx,&Nor_Vy);
	
//		c->hopemotorout.OmniSpeedOut[0] = -Vx+Vy+Vw;//右前
//		c->hopemotorout.OmniSpeedOut[1] = -Vx-Vy+Vw;//右后
//		c->hopemotorout.OmniSpeedOut[2] = Vx-Vy+Vw;//左后
//		c->hopemotorout.OmniSpeedOut[3] = Vx+Vy+Vw;//左前	
	if(Vw)
	{	
	  c->hopemotorout.OmniSpeedOut[0] = -Nor_Vx+Nor_Vy+Vw  ;//右前
		c->hopemotorout.OmniSpeedOut[1] = -Nor_Vx-Nor_Vy+Vw  ;//右后
		c->hopemotorout.OmniSpeedOut[2] = Nor_Vx-Nor_Vy+Vw   ;//左后
		c->hopemotorout.OmniSpeedOut[3] = Nor_Vx+Nor_Vy+Vw   ;//左前
	}
	else {
		c->hopemotorout.OmniSpeedOut[0] = -Nor_Vx+Nor_Vy+Vw  + c->pos088 .bmi088.gyro.z*gyro_kp;//右前
		c->hopemotorout.OmniSpeedOut[1] = -Nor_Vx-Nor_Vy+Vw  + c->pos088 .bmi088.gyro.z*gyro_kp;//右后
		c->hopemotorout.OmniSpeedOut[2] = Nor_Vx-Nor_Vy+Vw   + c->pos088 .bmi088.gyro.z*gyro_kp;//左后
		c->hopemotorout.OmniSpeedOut[3] = Nor_Vx+Nor_Vy+Vw   + c->pos088 .bmi088.gyro.z*gyro_kp;//左前
	}
//	
//	if(Vx || Vy){
//		PIAN = jiuzheng(c->pos088 .imu_eulr .yaw );	
//		c->hopemotorout.OmniSpeedOut[0] -= PIAN   ;//右前
//		c->hopemotorout.OmniSpeedOut[1] -= PIAN   ;//右后
//		c->hopemotorout.OmniSpeedOut[2] -= PIAN   ;//左后
//		c->hopemotorout.OmniSpeedOut[3] -= PIAN   ;//左前		
//	}
//	else 
//	{
//		PIAN = 0;
//		shang_yaw=c->pos088 .imu_eulr .yaw ;
//		pian_yaw=0;
//	}

//	
	c->vofa_send[0]=c->pos088.bmi088.gyro.z;
	
	c->vofa_send[1]=PIAN;
	c->vofa_send[2]=c->pos088.imu_eulr.yaw;
//	c->vofa_send[3]=
//	c->vofa_send[2]=c->motorfeedback .rotor_rpm3508 [1];
}


int8_t Chassis_Control(Chassis_t *c,CMD_t *ctrl,CAN_Output_t *out)
{
	if(c ==NULL)  return CHASSIS_ERR_NULL;
	if(ctrl ==NULL)  return CHASSIS_ERR_NULL;
	   

	  Chassis_SetCtrl(c,ctrl);
		 
	 //此处对imu加滤波做修正
	  c->pos088.bmi088.filtered_gyro.z =LowPassFilter2p_Apply(&(c->filled[0]),c->pos088.bmi088.gyro.z);
	  
	switch (c->chassis_ctrl .ctrl)
	{
	  case RCcontrol: //手动控制
		
/*     
	在cmd里对数据进行处理 包括方向和油门	  
	  6000为全向轮的倍率,遥控器坐标系和机器人坐标系不同*/
		if(c->chassis_ctrl .mode == Pitch){
	    c->move_vec.Vw = 0;
		  c->move_vec.Vx = 0;
		  c->move_vec.Vy = 0;
		}
		else if(c->chassis_ctrl .mode == UP_Adjust)
		{
	    c->move_vec.Vw = ctrl->Vw*6000;
		  c->move_vec.Vx = 0;
		  c->move_vec.Vy = 0;			
		}
		else {
	    c->move_vec.Vw = ctrl->Vw*6000;
		  c->move_vec.Vx = ctrl->Vy*6000;
		  c->move_vec.Vy = ctrl->Vx*6000;
		}
			
	

	  break;

        case AUTO: // 自动模式
            switch (c->chassis_ctrl.mode) {

                case AUTO_MID360: // 自动雷达
                    // 全向轮方向, 注意xy方向
                    c->move_vec.Vw = ctrl->cmd_MID360.posw * 1000;
                    c->move_vec.Vy = -ctrl->cmd_MID360.posy * 1000;
                    c->move_vec.Vx = -ctrl->cmd_MID360.posx * 1000;

                    abs_limit_fp(&c->move_vec.Vx, 5000.0f);
                    abs_limit_fp(&c->move_vec.Vy, 5000.0f);
                    abs_limit_fp(&c->move_vec.Vw, 5000.0f);

							      c->NUC_send .send [0]=0;
                    break;

                case AUTO_MID360_Pitch: 
                    c->move_vec.Vw = 0;
                    c->move_vec.Vy = 0;
                    c->move_vec.Vx = 0;
								
							     	c->NUC_send .send [0]=0;
                    break;
                case AUTO_MID360_Adjust: 
                    c->move_vec.Vw = ctrl->Vw * 6000;
                    c->move_vec.Vx = ctrl->Vy * 6000;
                    c->move_vec.Vy = ctrl->Vx * 6000;
								
							     	c->NUC_send .send [0]=1;
                    break;
                default:
                    c->move_vec.Vw = 0;
                    c->move_vec.Vx = 0;
                    c->move_vec.Vy = 0;
							     	c->NUC_send .send [0]=0;
								
                    break;
            }
            break;

        default:
            c->move_vec.Vw = 0;
            c->move_vec.Vx = 0;
            c->move_vec.Vy = 0;
            break;
    }

    // 电机速度限幅
    abs_limit_fp(&c->move_vec.Vx, 6000.0f);
    abs_limit_fp(&c->move_vec.Vy, 6000.0f);
    abs_limit_fp(&c->move_vec.Vw, 6000.0f);

  Chassis_speed_calculate(c,c->move_vec.Vx,c->move_vec.Vy,c->move_vec.Vw);
	
	
		for (uint8_t i = 0 ; i <4 ; i++)
	{
		 c->final_out.final_3508out[i] = PID_calc(&(c->pid.chassis_3508VecPID[i]),c->motorfeedback.rotor_rpm3508[i],c->hopemotorout.OmniSpeedOut[i]);
	 
		 out->motor_CHASSIS3508 .as_array[i] = c->final_out.final_3508out[i];
	}
	


//	c->vofa_send[0]=c->pos088.bmi088.gyro.z;
	
//	c->vofa_send[1]=c->motorfeedback .rotor_rpm3508 [0];
//	c->vofa_send[2]=c->motorfeedback .rotor_rpm3508 [1];
//	c->vofa_send[3]=c->motorfeedback .rotor_rpm3508 [2];
//  c->vofa_send[4]=c->motorfeedback .rotor_rpm3508 [3];
//	
//  c->vofa_send[0]=c->hopemotorout .OmniSpeedOut [0];
//  c->vofa_send[1]=c->hopemotorout .OmniSpeedOut [0];
////  c->vofa_send[2]=c->hopemotorout .OmniSpeedOut [2];
//  c->vofa_send[3]=c->hopemotorout .OmniSpeedOut [3];

////	c->vofa_send[4]=c->motorfeedback .rotor_rpm3508 [0];
//	c->vofa_send[0]=c->motorfeedback .rotor_rpm3508 [0];
////	c->vofa_send[6]=c->motorfeedback .rotor_rpm3508 [2];
//  c->vofa_send[2]=c->motorfeedback .rotor_rpm3508 [3];	
//	
//	  c->vofa_send[5]=c->pos088 .imu_eulr .yaw  ;

	return CHASSIS_OK;

}
	 pid_type_def pid;

	pid_param_t pid_param={ 
		.p = 0.50f,        
    .i = 0.0f,         
    .d = 0.0f,         
    .i_limit = 50.0f,
    .out_limit = 100.0f
	};
fp32  jiuzheng(fp32 yaw)
{
	static int is=0;

	
	if(is==0)
	{
		PID_init (&pid,PID_POSITION ,&pid_param);
		is=1;
	}
	
	pian_yaw+= (yaw - shang_yaw)*1000;	
	shang_yaw=yaw ;
	
	return PID_calc (&pid,pian_yaw,0);
}

